/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021-2022, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_winix_core_header
#define headerfile_winix_core_header

#include "log.h"


namespace Winix
{

class Header
{
public:

	/*
	 * headers' names
	 */
	static constexpr const wchar_t * content_type 				= L"Content-Type";
	static constexpr const wchar_t * accept 					= L"Accept";
	static constexpr const wchar_t * accept_language			= L"Accept-Language";

	/*
	 * headers' names lower case
	 */
	static constexpr const wchar_t * content_type_lc 			= L"content-type";
	static constexpr const wchar_t * accept_lc					= L"accept";
	static constexpr const wchar_t * accept_language_lc			= L"accept-language";

	/*
	 * headers' values
	 */
	static constexpr const wchar_t * application_octet_stream 	= L"application/octet-stream";

	static constexpr const wchar_t * text_html					= L"text/html";
	static constexpr const wchar_t * application_json			= L"application/json";
	static constexpr const wchar_t * application_xml			= L"application/xml";
	static constexpr const wchar_t * application_xhtml_xml		= L"application/xhtml+xml";
	static constexpr const wchar_t * text_csv					= L"text/csv";
	static constexpr const wchar_t * text_javascript			= L"text/javascript";

	static constexpr const wchar_t * all_all					= L"*/*";
	static constexpr const wchar_t * text_all					= L"text/*";
	static constexpr const wchar_t * application_all			= L"application/*";

	static constexpr const wchar_t * text_html_utf8				= L"text/html; charset=UTF-8";
	static constexpr const wchar_t * application_json_utf8		= L"application/json; charset=UTF-8";
	static constexpr const wchar_t * application_xml_utf8		= L"application/xml; charset=UTF-8";
	static constexpr const wchar_t * application_xhtml_xml_utf8	= L"application/xhtml+xml; charset=UTF-8";
	static constexpr const wchar_t * text_csv_utf8				= L"text/csv; charset=UTF-8";
	static constexpr const wchar_t * text_javascript_utf8		= L"text/javascript; charset=UTF-8";


};


class HeaderValue
{
public:

	std::wstring value;
	double weight; // q parameter in some headers


	HeaderValue()
	{
		weight = 0.0;
	}


	static void log_values(const std::vector<HeaderValue> & header_values, Log & log)
	{
		char buf[64];
		bool is_first = true;

		for(const HeaderValue & h: header_values)
		{
			snprintf(buf, sizeof(buf)/sizeof(char), "%.3f", h.weight);

			if( !is_first )
				log << ", ";

			log << h.value << ";q=" << buf;
			is_first = false;
		}
	}

};

}

#endif

